@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed-single.cpp,v 1.92.4.3 2004/10/07 11:15:47 marco Exp $
+ *  $Id: mozilla-embed-single.cpp,v 1.92.4.4 2004/10/10 16:18:26 marco Exp $
  */
 
 #include "mozilla-config.h"
@@ -627,6 +627,12 @@ impl_get_offline_mode (EphyEmbedSingle *shell)
 	return isOffline;
 }
 
+static GList *
+impl_get_printer_list (EphyEmbedSingle *shell)
+{
+	return MozillaPrivate::GetPrinterList ();
+}
+
 static GList *
 impl_get_font_list (EphyEmbedSingle *shell,
 		    const char *langGroup)
@@ -959,7 +965,7 @@ impl_open_window (EphyEmbedSingle *single,
 	wWatch->OpenWindow (domWindow, address, name, features, nsnull,
 			    getter_AddRefs (newWindow));
 
-	return MozillaFindEmbed (newWindow);
+	return EphyUtils::FindEmbed (newWindow);
 }
 
 static void
@@ -984,6 +990,7 @@ ephy_embed_single_iface_init (EphyEmbedSingleIface *iface)
 	iface->get_offline_mode = impl_get_offline_mode;
 	iface->get_font_list = impl_get_font_list;
 	iface->open_window = impl_open_window;
+	iface->get_printer_list = impl_get_printer_list;
 }
 
 static void
