@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2000-2003 Marco Pesenti Gritti
+ *  Copyright (C) 2000-2004 Marco Pesenti Gritti
+ *  Copyright (C) 2003, 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,9 +16,11 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed.cpp,v 1.62.4.4 2004/04/28 18:09:59 chpe Exp $
+ *  $Id: mozilla-embed.cpp,v 1.83.4.3 2004/10/10 16:18:26 marco Exp $
  */
 
+#include "mozilla-config.h"
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -29,28 +32,24 @@
 #include "ephy-string.h"
 #include "ephy-debug.h"
 
-#include "MozillaPrivate.h"
 #include "EphyBrowser.h"
 #include "EventContext.h"
+#include "EphyUtils.h"
 
 #include <gtkmozembed.h>
-#include <nsIWindowWatcher.h>
+#include <nsEmbedString.h>
+#include <nsMemory.h>
 #include <nsIURI.h>
-#include <nsIURL.h>
-#include <nsNetUtil.h>
-#include <nsString.h>
 #include <nsIRequest.h>
 #include <nsIWebProgressListener.h>
-#include <nsITransportSecurityInfo.h>
-#include <nsIPrintOptions.h>
 #include <nsGfxCIID.h>
 
 static void	mozilla_embed_class_init	(MozillaEmbedClass *klass);
 static void	mozilla_embed_init		(MozillaEmbed *gs);
 static void	mozilla_embed_destroy		(GtkObject *object);
+static void	mozilla_embed_finalize		(GObject *object);
 static void	ephy_embed_iface_init		(EphyEmbedIface *iface);
 
-static void mozilla_embed_connect_signals	(MozillaEmbed *membed);
 static void mozilla_embed_location_changed_cb	(GtkMozEmbed *embed,
 						 MozillaEmbed *membed);
 static void mozilla_embed_net_state_all_cb	(GtkMozEmbed *embed,
@@ -78,17 +77,23 @@ static EmbedSecurityLevel mozilla_embed_security_level (MozillaEmbed *membed);
 
 #define MOZILLA_EMBED_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), MOZILLA_TYPE_EMBED, MozillaEmbedPrivate))
 
+typedef enum
+{
+	MOZILLA_EMBED_LOAD_STARTED,
+	MOZILLA_EMBED_LOAD_REDIRECTING,
+	MOZILLA_EMBED_LOAD_LOADING,
+	MOZILLA_EMBED_LOAD_STOPPED
+} MozillaEmbedLoadState;
+
 struct MozillaEmbedPrivate
 {
 	EphyBrowser *browser;
-	nsCOMPtr<nsIRequest> request;
-	gint security_state;
+	guint security_state;
+	MozillaEmbedLoadState load_state;
 };
 
 #define WINDOWWATCHER_CONTRACTID "@mozilla.org/embedcomp/window-watcher;1"
 
-static NS_DEFINE_CID(kPrintOptionsCID, NS_PRINTOPTIONS_CID);
-
 static GObjectClass *parent_class = NULL;
 
 static void
@@ -105,12 +110,12 @@ impl_manager_can_do_command (EphyCommandManager *manager,
 			     const char *command) 
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(manager)->priv;
-	nsresult result;
+	nsresult rv;
 	PRBool enabled;
 
-        result = mpriv->browser->GetCommandState (command, &enabled);
+        rv = mpriv->browser->GetCommandState (command, &enabled);
 
-	return NS_SUCCEEDED (result) ? enabled : FALSE;
+	return NS_SUCCEEDED (rv) ? enabled : FALSE;
 }
 
 static void
@@ -174,12 +179,12 @@ impl_find_next (EphyEmbed *embed,
                 gboolean backwards)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
-	nsresult result;
+	nsresult rv;
         PRBool didFind;
 
-        result = mpriv->browser->Find (backwards, &didFind);
+        rv = mpriv->browser->Find (backwards, &didFind);
 	
-	return NS_SUCCEEDED (result) ? didFind : FALSE;
+	return NS_SUCCEEDED (rv) ? didFind : FALSE;
 }
 
 static void
@@ -190,15 +195,18 @@ impl_activate (EphyEmbed *embed)
 
 static void
 impl_find_set_properties (EphyEmbed *embed, 
-                          char *search_string,
+                          const char *search_string,
 	                  gboolean case_sensitive,
 			  gboolean wrap_around)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
 
-	mpriv->browser->FindSetProperties
-		((NS_ConvertUTF8toUCS2(search_string)).get(),
-		 case_sensitive, wrap_around); 
+	nsEmbedString searchString;
+	NS_CStringToUTF16 (nsEmbedCString(search_string),
+			   NS_CSTRING_ENCODING_UTF8, searchString);
+
+	mpriv->browser->FindSetProperties (searchString.get(), case_sensitive,
+					   wrap_around); 
 }
 
 static void
@@ -208,10 +216,10 @@ mozilla_embed_realize (GtkWidget *widget)
 
 	(* GTK_WIDGET_CLASS(parent_class)->realize) (widget);
 
-	nsresult result;
-	result = mpriv->browser->Init (GTK_MOZ_EMBED (widget));
+	nsresult rv;
+	rv = mpriv->browser->Init (GTK_MOZ_EMBED (widget));
 
-	if (NS_FAILED(result))
+	if (NS_FAILED (rv))
 	{
                	g_warning ("EphyBrowser initialization failed for %p\n", widget);
 	}
@@ -238,6 +246,7 @@ mozilla_embed_class_init (MozillaEmbedClass *klass)
 	parent_class = (GObjectClass *) g_type_class_peek_parent (klass);
 
 	object_class->constructor = mozilla_embed_constructor;
+	object_class->finalize = mozilla_embed_finalize;
 
 	gtk_object_class->destroy = mozilla_embed_destroy;
 
@@ -251,22 +260,8 @@ mozilla_embed_init (MozillaEmbed *embed)
 {
         embed->priv = MOZILLA_EMBED_GET_PRIVATE (embed);
 	embed->priv->browser = new EphyBrowser ();
-	embed->priv->security_state = -1;
+	embed->priv->security_state = STATE_IS_UNKNOWN;
 
-	mozilla_embed_connect_signals (embed);
-}
-
-gpointer
-_mozilla_embed_get_ephy_browser (MozillaEmbed *embed)
-{
-	g_return_val_if_fail (embed->priv->browser != NULL, NULL);
-	
-	return embed->priv->browser;
-}
-
-static void 
-mozilla_embed_connect_signals (MozillaEmbed *embed)
-{
 	g_signal_connect_object (G_OBJECT (embed), "location",
 				 G_CALLBACK (mozilla_embed_location_changed_cb),
 				 embed, (GConnectFlags) 0);
@@ -290,22 +285,41 @@ mozilla_embed_connect_signals (MozillaEmbed *embed)
 				 embed, (GConnectFlags) 0);
 }
 
+gpointer
+_mozilla_embed_get_ephy_browser (MozillaEmbed *embed)
+{
+	g_return_val_if_fail (embed->priv->browser != NULL, NULL);
+	
+	return embed->priv->browser;
+}
+
 static void
 mozilla_embed_destroy (GtkObject *object)
 {
 	MozillaEmbed *embed = MOZILLA_EMBED (object);
-	int i;
 
 	if (embed->priv->browser)
 	{
 		embed->priv->browser->Destroy();
-        	delete embed->priv->browser;
-        	embed->priv->browser = NULL;
 	}
 	
 	GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
 
+static void
+mozilla_embed_finalize (GObject *object)
+{
+	MozillaEmbed *embed = MOZILLA_EMBED (object);
+
+	if (embed->priv->browser)
+	{
+        	delete embed->priv->browser;
+	       	embed->priv->browser = nsnull;
+	}
+
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
 static void
 impl_load_url (EphyEmbed *embed, 
                const char *url)
@@ -337,44 +351,24 @@ mozilla_embed_get_uri_parent (const char *aUri)
         nsresult rv;
 
         nsCOMPtr<nsIURI> uri;
-        rv = NS_NewURI (getter_AddRefs(uri), aUri);
+        rv = EphyUtils::NewURI (getter_AddRefs(uri), nsEmbedCString(aUri));
         if (NS_FAILED(rv) || !uri) return NULL;
 
-        nsCOMPtr<nsIURL> url = do_QueryInterface(uri, &rv);
-        if (NS_FAILED(rv) || !url) return NULL;
+	nsEmbedCString path;
+	rv = uri->GetPath(path);
+	if (NS_FAILED(rv)) return NULL;
 
-        nsCAutoString dirPath;
-        rv = url->GetDirectory (dirPath);
-        if (NS_FAILED(rv) || !dirPath.Length()) return NULL;
+	if (!path.Length() || strcmp (path.get(), "/") == 0 ||
+	    !strchr (path.get(), '/'))
+	{
+		return NULL;
+	}
 
-        nsCAutoString filePath;
-        rv = url->GetFilePath (filePath);
-        if (NS_FAILED(rv) || !filePath.Length()) return NULL;
+	nsEmbedCString parent;
+	rv = uri->Resolve (nsEmbedCString(".."), parent);
+	if (NS_FAILED(rv)) return NULL;
 
-        PRInt32 pathLength = filePath.Length();
-        PRInt32 trailingSlash = filePath.RFind("/");
-
-        if(pathLength < 2 || trailingSlash == -1)
-        {
-                return NULL;
-        }
-
-        if(trailingSlash != (pathLength-1))
-        {
-                uri->SetPath(dirPath);
-        }
-        else
-        {
-                PRInt32 nextSlash = filePath.RFind("/",PR_FALSE,trailingSlash-1);
-                nsCAutoString parentPath;
-                filePath.Left(parentPath, nextSlash);
-                uri->SetPath(parentPath);
-        }
-
-        nsCAutoString spec;
-        uri->GetSpec(spec);
-
-        return !spec.IsEmpty() ? g_strdup(spec.get()) : NULL;
+	return g_strdup (parent.get()); 
 }
 
 static gboolean
@@ -446,7 +440,10 @@ impl_go_up (EphyEmbed *embed)
 	char *parent_uri;
 
 	uri = ephy_embed_get_location (embed, TRUE);
-	g_return_if_fail (uri != NULL);
+	if (uri == NULL)
+	{
+		return;
+	}
 	
 	parent_uri = mozilla_embed_get_uri_parent (uri);
 	g_free (uri);
@@ -483,18 +480,18 @@ impl_get_location (EphyEmbed *embed,
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
 	char *l;
 	nsresult rv;
-	nsCAutoString url;
+	nsEmbedCString url;
 
 	if (toplevel)
 	{
 		rv = mpriv->browser->GetDocumentUrl (url);
-		l = (NS_SUCCEEDED (rv) && !url.IsEmpty()) ?
+		l = (NS_SUCCEEDED (rv) && url.Length()) ?
 		     g_strdup (url.get()) : NULL;	   	
 	}
 	else
 	{
 		rv = mpriv->browser->GetTargetDocumentUrl (url);
-		l = (NS_SUCCEEDED (rv) && !url.IsEmpty()) ?
+		l = (NS_SUCCEEDED (rv) && url.Length()) ?
 		     g_strdup (url.get()) : NULL;	   	
 	}
 
@@ -503,51 +500,60 @@ impl_get_location (EphyEmbed *embed,
 
 static void
 impl_reload (EphyEmbed *embed, 
-             EmbedReloadFlags flags)
+             gboolean force)
 {
+#ifndef GTKMOZEMBED_BROKEN_RELOAD
 	guint32 mflags;
 
 	mflags = GTK_MOZ_EMBED_FLAG_RELOADNORMAL;
 
-	if (flags & EMBED_RELOAD_FORCE)
+	if (force)
 	{
 		mflags = GTK_MOZ_EMBED_FLAG_RELOADBYPASSPROXYANDCACHE;
 	}
-	
-	gtk_moz_embed_reload (GTK_MOZ_EMBED(embed),
-			      mflags);
+
+	gtk_moz_embed_reload (GTK_MOZ_EMBED(embed), mflags);
+#else
+	/* Workaround for broken reload with frames, see mozilla bug
+	 * http://bugzilla.mozilla.org/show_bug.cgi?id=246392
+	 */
+	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED (embed)->priv;
+
+	mpriv->browser->Reload (force ? EphyBrowser::RELOAD_FORCE :
+					EphyBrowser::RELOAD_NORMAL);
+#endif
 }
 
 static void
-impl_zoom_set (EphyEmbed *embed, 
-               float zoom, 
-               gboolean reflow)
+impl_set_zoom (EphyEmbed *embed, 
+               float zoom) 
 {
 	EphyBrowser *browser;
-	nsresult result;
+	nsresult rv;
 
 	g_return_if_fail (zoom > 0.0);
 
 	browser = MOZILLA_EMBED(embed)->priv->browser;
 	g_return_if_fail (browser != NULL);
 
-	result = browser->SetZoom (zoom, reflow);
+	rv = browser->SetZoom (zoom);
 
-	if (NS_SUCCEEDED (result))
+	if (NS_SUCCEEDED (rv))
 	{
 		g_signal_emit_by_name (embed, "ge_zoom_change", zoom);
 	}
 }
 
 static float
-impl_zoom_get (EphyEmbed *embed)
+impl_get_zoom (EphyEmbed *embed)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
 	float f;
+
+	nsresult rv;	
+	rv = mpriv->browser->GetZoom (&f);
 	
-	nsresult result = mpriv->browser->GetZoom (&f);
-	
-	if (NS_SUCCEEDED (result))
+	if (NS_SUCCEEDED (rv))
 	{
 		return f;
 	}
@@ -575,7 +581,7 @@ impl_shistory_get_nth (EphyEmbed *embed,
                        char **aTitle)
 {
 	nsresult rv;
-        nsCAutoString url;
+        nsEmbedCString url;
 	PRUnichar *title;
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
 
@@ -586,13 +592,22 @@ impl_shistory_get_nth (EphyEmbed *embed,
 	
         rv = mpriv->browser->GetSHUrlAtIndex(nth, url);
 
-        *aUrl = (NS_SUCCEEDED (rv) && !url.IsEmpty()) ? g_strdup(url.get()) : NULL;
+        *aUrl = (NS_SUCCEEDED (rv) && url.Length()) ? g_strdup(url.get()) : NULL;
 
 	rv = mpriv->browser->GetSHTitleAtIndex(nth, &title);
 
-	*aTitle = g_strdup (NS_ConvertUCS2toUTF8(title).get());
-
-	nsMemory::Free (title);
+	if (title)
+	{
+		nsEmbedCString cTitle;
+		NS_UTF16ToCString (nsEmbedString(title),
+				   NS_CSTRING_ENCODING_UTF8, cTitle);
+		*aTitle = g_strdup (cTitle.get());
+		nsMemory::Free (title);
+	}
+	else
+	{
+		*aTitle = NULL;
+	}
 }
 
 static int
@@ -621,85 +636,48 @@ impl_get_security_level (EphyEmbed *embed,
                          EmbedSecurityLevel *level,
                          char **description)
 {
-	nsresult result;
+	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED (embed)->priv;
+
+	g_return_if_fail (description != NULL && level != NULL);
 
-	g_return_if_fail (description != NULL || level != NULL);
 	*description = NULL;
 	*level = STATE_IS_UNKNOWN;
 
-        nsCOMPtr<nsIChannel> channel;
-	channel = do_QueryInterface (MOZILLA_EMBED(embed)->priv->request, 
-				     &result);
-        if (NS_FAILED (result)) return;
+	nsresult rv;
+	nsEmbedCString desc;
+	rv = mpriv->browser->GetSecurityDescription (desc);
+	if (NS_FAILED (rv)) return;
 
-        nsCOMPtr<nsISupports> info;
-        result = channel->GetSecurityInfo(getter_AddRefs(info));
-        if (NS_FAILED (result)) return;
-
-	if (info)
-	{
-		nsCOMPtr<nsITransportSecurityInfo> secInfo(do_QueryInterface(info));
-		if (!secInfo) return;
-
-		nsXPIDLString tooltip;
-		result = secInfo->GetShortSecurityDescription(getter_Copies(tooltip));
-		if (NS_FAILED (result)) return;
-
-		if (tooltip)
-		{
-			*description = g_strdup (NS_ConvertUCS2toUTF8(tooltip).get());
-		}
-	}
-	
+	*description = g_strdup (desc.get());
 	*level = mozilla_embed_security_level (MOZILLA_EMBED (embed));
 }
 
 static void
-impl_print (EphyEmbed *embed,
-            EmbedPrintInfo *info)
+impl_print (EphyEmbed *embed)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
-	nsresult result;
-
-        nsCOMPtr<nsIPrintSettings> options;
-        result = mpriv->browser->GetPrintSettings(getter_AddRefs(options));
-        if (NS_FAILED (result) || !options) return;
-
-	/* work around mozilla bug which borks when printing selection without having one */
-	if (info->pages == 2 && ephy_command_manager_can_do_command
-		(EPHY_COMMAND_MANAGER (embed), "cmd_copy") == FALSE)
-	{
-		info->pages = 0;
-	}
-
-	MozillaCollatePrintSettings(info, options);
-
-        options->SetPrintSilent (PR_TRUE);
-
-	result = mpriv->browser->Print(options, info->preview);
-
-	/* Workaround for bug 125984 */
-        options->SetPrintSilent (PR_FALSE);
+ 
+	mpriv->browser->Print ();
 }
 
 static void
-impl_print_preview_close (EphyEmbed *embed)
+impl_set_print_preview_mode (EphyEmbed *embed, gboolean preview_mode)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
 
-	mpriv->browser->PrintPreviewClose();
+	mpriv->browser->SetPrintPreviewMode (preview_mode);
 }
 
 static int
 impl_print_preview_n_pages (EphyEmbed *embed)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
-	nsresult result;
+	nsresult rv;
 	int num;
 
-	result = mpriv->browser->PrintPreviewNumPages(&num);
+	rv = mpriv->browser->PrintPreviewNumPages(&num);
 
-	return NS_SUCCEEDED(result) ? num : 0;
+	return NS_SUCCEEDED (rv) ? num : 0;
 }
 
 static void
@@ -708,9 +686,8 @@ impl_print_preview_navigate (EphyEmbed *embed,
 			     int page)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
-	nsresult result;
 
-	result = mpriv->browser->PrintPreviewNavigate(type, page);
+	mpriv->browser->PrintPreviewNavigate(type, page);
 }
 
 static void
@@ -718,44 +695,76 @@ impl_set_encoding (EphyEmbed *embed,
 		   const char *encoding)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
-	nsresult result;
+	nsresult rv;
+	nsEmbedCString currEnc;
 
-	result = mpriv->browser->ForceEncoding (encoding);
-	if (NS_FAILED (result)) return;
+	g_return_if_fail (encoding != NULL);
 
+	rv = mpriv->browser->GetEncoding (currEnc);
+	if (NS_FAILED (rv)) return;
+
+	if (strcmp (currEnc.get(), encoding) != 0 ||
+	    encoding[0] == '\0' && !ephy_embed_has_automatic_encoding (embed))
+	{
+		rv = mpriv->browser->ForceEncoding (encoding);
+		if (NS_FAILED (rv)) return;
+	}
+
+#ifndef GTKMOZEMBED_BROKEN_RELOAD
 	gtk_moz_embed_reload (GTK_MOZ_EMBED (embed),
 			      GTK_MOZ_EMBED_FLAG_RELOADCHARSETCHANGE);
+#else
+	/* Workaround for broken reload with frames, see mozilla bug
+	 * http://bugzilla.mozilla.org/show_bug.cgi?id=246392
+	 */
+	mpriv->browser->Reload (EphyBrowser::RELOAD_ENCODING_CHANGE);
+#endif
 }
 
-static EphyEncodingInfo *
-impl_get_encoding_info (EphyEmbed *embed)
+static char *
+impl_get_encoding (EphyEmbed *embed)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
-	nsresult result;
-	EphyEncodingInfo *info = NULL;
+	nsresult rv;
+	nsEmbedCString encoding;
 
-	result = mpriv->browser->GetEncodingInfo (&info);
+	rv = mpriv->browser->GetEncoding (encoding);
 
-	if (NS_FAILED (result))
+	if (NS_FAILED (rv) || !encoding.Length())
 	{
-		ephy_encoding_info_free (info);
-
 		return NULL;
 	}
 
-	return info;
+	return g_strdup (encoding.get());
+}
+
+static gboolean
+impl_has_automatic_encoding (EphyEmbed *embed)
+{
+	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
+	nsresult rv;
+	nsEmbedCString encoding;
+
+	rv = mpriv->browser->GetForcedEncoding (encoding);
+
+	if (NS_FAILED (rv) || !encoding.Length())
+	{
+		return TRUE;
+	}
+
+	return FALSE;
 }
 
 static gboolean
 impl_has_modified_forms (EphyEmbed *embed)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
-	nsresult result;
+	nsresult rv;
 
 	PRBool modified;
-	result = mpriv->browser->GetHasModifiedForms (&modified);
+	rv = mpriv->browser->GetHasModifiedForms (&modified);
 
-	return modified == PR_TRUE ? TRUE : FALSE;
+	return NS_SUCCEEDED (rv) ? modified : FALSE;
 }
 
 static void
@@ -769,6 +778,46 @@ mozilla_embed_location_changed_cb (GtkMozEmbed *embed,
 	g_free (location);
 }
 
+static void
+update_load_state (MozillaEmbed *membed, gint state)
+{
+	MozillaEmbedPrivate *priv = membed->priv;
+
+	if (state & GTK_MOZ_EMBED_FLAG_IS_NETWORK)
+	{
+		if (state & GTK_MOZ_EMBED_FLAG_START)
+		{
+			priv->load_state = MOZILLA_EMBED_LOAD_STARTED;
+		}
+		else if (state & GTK_MOZ_EMBED_FLAG_STOP)
+		{
+			priv->load_state = MOZILLA_EMBED_LOAD_STOPPED;
+		}
+	}
+	else if (state & GTK_MOZ_EMBED_FLAG_START &&
+	         state & GTK_MOZ_EMBED_FLAG_IS_REQUEST)
+	{
+		if (priv->load_state == MOZILLA_EMBED_LOAD_REDIRECTING)
+		{
+			priv->load_state = MOZILLA_EMBED_LOAD_STARTED;
+		}
+		else if (priv->load_state != MOZILLA_EMBED_LOAD_LOADING)
+		{
+			priv->load_state = MOZILLA_EMBED_LOAD_LOADING;
+
+			char *address;
+			address = gtk_moz_embed_get_location (GTK_MOZ_EMBED (membed));
+			g_signal_emit_by_name (membed, "ge_content_change", address);
+			g_free (address);
+		}
+	}
+	else if (state & GTK_MOZ_EMBED_FLAG_REDIRECTING &&
+	         priv->load_state == MOZILLA_EMBED_LOAD_STARTED)
+	{
+		priv->load_state = MOZILLA_EMBED_LOAD_REDIRECTING;
+	}
+}
+
 static void
 mozilla_embed_net_state_all_cb (GtkMozEmbed *embed, const char *aURI,
                                 gint state, guint status, 
@@ -776,7 +825,7 @@ mozilla_embed_net_state_all_cb (GtkMozEmbed *embed, const char *aURI,
 {
 	EmbedState estate = EMBED_STATE_UNKNOWN;
 	int i;
-	
+
 	struct
 	{
 		guint state;
@@ -802,6 +851,8 @@ mozilla_embed_net_state_all_cb (GtkMozEmbed *embed, const char *aURI,
 			estate = (EmbedState) (estate | conversion_map[i].embed_state);	
 		}
 	}
+
+	update_load_state (membed, state);
 	
 	g_signal_emit_by_name (membed, "ge_net_state", aURI, estate);
 }
@@ -837,10 +888,8 @@ mozilla_embed_dom_key_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 	if ((info->keycode == nsIDOMKeyEvent::DOM_VK_F10 &&
 	    (info->modifier == GDK_SHIFT_MASK ||
 	     info->modifier == GDK_CONTROL_MASK))
-#if MOZILLA_SNAPSHOT > 14
 	    || (info->keycode == nsIDOMKeyEvent::DOM_VK_CONTEXT_MENU &&
 		!info->modifier)
-#endif
 	   )
 	{
 		/* Translate relative coordinates to absolute values, and try
@@ -880,7 +929,7 @@ mozilla_embed_dom_mouse_click_cb (GtkMozEmbed *embed, gpointer dom_event,
 	MozillaEmbedEvent *info;
 	EventContext event_context;
 	gint return_value = FALSE;
-	nsresult result;
+	nsresult rv;
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
 
 	if (dom_event == NULL)
@@ -897,16 +946,16 @@ mozilla_embed_dom_mouse_click_cb (GtkMozEmbed *embed, gpointer dom_event,
 	info = mozilla_embed_event_new (NS_STATIC_CAST (gpointer, dev));
 
 	event_context.Init (mpriv->browser);
-        result = event_context.GetMouseEventInfo (ev, MOZILLA_EMBED_EVENT (info));
+        rv = event_context.GetMouseEventInfo (ev, MOZILLA_EMBED_EVENT (info));
 
-	if (NS_SUCCEEDED(result))
+	if (NS_SUCCEEDED (rv))
 	{
 		nsCOMPtr<nsIDOMDocument> domDoc;
-		result = event_context.GetTargetDocument (getter_AddRefs(domDoc));
-		if (NS_SUCCEEDED(result))
+		rv = event_context.GetTargetDocument (getter_AddRefs(domDoc));
+		if (NS_SUCCEEDED (rv))
 		{
-			result = mpriv->browser->PushTargetDocument (domDoc);
-			if (NS_SUCCEEDED(result))
+			rv = mpriv->browser->PushTargetDocument (domDoc);
+			if (NS_SUCCEEDED (rv))
 			{
 				g_signal_emit_by_name (membed, "ge_dom_mouse_click", 
 						       info, &return_value); 
@@ -928,7 +977,7 @@ mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 	MozillaEmbedEvent *info;
 	EventContext event_context;
 	gint return_value = FALSE;
-	nsresult result;
+	nsresult rv;
 	EphyEmbedEventType type;
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
 
@@ -946,18 +995,18 @@ mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 	info = mozilla_embed_event_new (NS_STATIC_CAST (gpointer, dev));
 
 	event_context.Init (mpriv->browser);
-        result = event_context.GetMouseEventInfo (ev, MOZILLA_EMBED_EVENT (info));
-	if (NS_FAILED (result)) return FALSE;
+        rv = event_context.GetMouseEventInfo (ev, MOZILLA_EMBED_EVENT (info));
+	if (NS_FAILED (rv)) return FALSE;
 
 	type = ephy_embed_event_get_event_type ((EphyEmbedEvent *) info);
 		
 	nsCOMPtr<nsIDOMDocument> domDoc;
-	result = event_context.GetTargetDocument (getter_AddRefs(domDoc));
-	if (NS_SUCCEEDED(result))
+	rv = event_context.GetTargetDocument (getter_AddRefs(domDoc));
+	if (NS_SUCCEEDED (rv))
 	{
-		result = mpriv->browser->PushTargetDocument (domDoc);
+		rv = mpriv->browser->PushTargetDocument (domDoc);
 
-		if (NS_SUCCEEDED(result))
+		if (NS_SUCCEEDED (rv))
 		{
 			g_signal_emit_by_name (membed, "ge_dom_mouse_down", 
 					       info, &return_value); 
@@ -1025,13 +1074,13 @@ mozilla_embed_new_window_cb (GtkMozEmbed *embed,
 
 static void
 mozilla_embed_security_change_cb (GtkMozEmbed *embed, 
-				  gpointer request,
-                                  guint state, 
+				  gpointer requestptr,
+				  guint state, 
 				  MozillaEmbed *membed)
 {
 	EmbedSecurityLevel level;
 
-	membed->priv->request = static_cast<nsIRequest*>(request);
+	membed->priv->browser->SetSecurityInfo (static_cast<nsIRequest*>(requestptr));
 	membed->priv->security_state = state;
 	level = mozilla_embed_security_level (membed);
 
@@ -1087,8 +1136,8 @@ ephy_embed_iface_init (EphyEmbedIface *iface)
 	iface->get_link_message = impl_get_link_message;
 	iface->get_js_status = impl_get_js_status;
 	iface->reload = impl_reload;
-	iface->zoom_set = impl_zoom_set;
-	iface->zoom_get = impl_zoom_get;
+	iface->set_zoom = impl_set_zoom;
+	iface->get_zoom = impl_get_zoom;
 	iface->shistory_n_items = impl_shistory_n_items;
 	iface->shistory_get_nth = impl_shistory_get_nth;
 	iface->shistory_get_pos = impl_shistory_get_pos;
@@ -1098,15 +1147,16 @@ ephy_embed_iface_init (EphyEmbedIface *iface)
 	iface->activate = impl_activate;
 	iface->find_set_properties = impl_find_set_properties;
 	iface->set_encoding = impl_set_encoding;
-	iface->get_encoding_info = impl_get_encoding_info;
+	iface->get_encoding = impl_get_encoding;
+	iface->has_automatic_encoding = impl_has_automatic_encoding;
 	iface->print = impl_print;
-	iface->print_preview_close = impl_print_preview_close;
+	iface->set_print_preview_mode = impl_set_print_preview_mode;
 	iface->print_preview_n_pages = impl_print_preview_n_pages;
 	iface->print_preview_navigate = impl_print_preview_navigate;
 	iface->has_modified_forms = impl_has_modified_forms;
 }
 
-void
+static void
 xul_visibility_cb (GtkWidget *embed, gboolean visibility, GtkWidget *window)
 {
 	if (visibility)
@@ -1119,12 +1169,23 @@ xul_visibility_cb (GtkWidget *embed, gboolean visibility, GtkWidget *window)
 	}
 }
 
-void
+static void
 xul_size_to_cb (GtkWidget *embed, gint width, gint height, gpointer dummy)
 {
 	gtk_widget_set_size_request (embed, width, height);
 }
 
+static void
+xul_new_window_cb (GtkMozEmbed *embed,
+		   GtkMozEmbed **retval, 
+		   guint chrome_mask,
+		   gpointer dummy)
+{
+        g_assert (chrome_mask & GTK_MOZ_EMBED_FLAG_OPENASCHROME);
+
+        *retval = _mozilla_embed_new_xul_dialog ();
+}
+
 GtkMozEmbed *
 _mozilla_embed_new_xul_dialog (void)
 {
@@ -1144,6 +1205,9 @@ _mozilla_embed_new_xul_dialog (void)
 	g_signal_connect_object (embed, "size_to",
 				 G_CALLBACK (xul_size_to_cb),
 				 NULL, (GConnectFlags) 0);
+	g_signal_connect_object (embed, "new_window",
+				 G_CALLBACK (xul_new_window_cb),
+				 NULL, (GConnectFlags) 0);
 
 	return GTK_MOZ_EMBED (embed);
 }
