@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-dialog.c,v 1.37 2004/06/29 12:38:29 chpe Exp $
+ *  $Id: ephy-dialog.c,v 1.37.4.3 2004/10/10 16:18:26 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -465,7 +465,7 @@ static void
 set_editable_from_value (PropertyInfo *info, const GValue *value)
 {
 	char *text = NULL;
-	int pos;
+	int pos = 0; /* insertion position */
 
 	g_return_if_fail (GTK_IS_EDITABLE (info->widget));
 
@@ -587,16 +587,10 @@ set_combo_box_from_value (PropertyInfo *info, const  GValue *value)
 		}
 		else
 		{
-			char *v;
-
-			gtk_combo_box_set_active (GTK_COMBO_BOX (info->widget), -1);
-
-			v = g_strdup_value_contents (value);
-			g_warning ("Value '%s' not found in model for combo %s\n", v, info->id);
-			g_free (v);
+			gtk_combo_box_set_active (GTK_COMBO_BOX (info->widget), 0);
 		}
 
-		info->sane_state = found;
+		info->sane_state = TRUE;
 	}
 	else if (info->data_type == G_TYPE_INT)
 	{
@@ -1018,7 +1012,7 @@ setup_default_size (EphyDialog *dialog)
 				       dialog->priv->name,
 				       dialog->priv->default_width,
 				       dialog->priv->default_height,
-				       flags);
+				       FALSE, flags);
 
 		dialog->priv->has_default_size = TRUE;
 	}
