@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-gui.c,v 1.22.4.1 2004/10/04 22:03:23 chpe Exp $
+ *  $Id: ephy-gui.c,v 1.22.4.2 2004/10/07 13:29:24 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -38,6 +38,7 @@
 #include <gtk/gtkmain.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtkversion.h>
+#include <gtk/gtkicontheme.h>
 
 /* Styles for tab labels */
 GtkStyle *loading_text_style = NULL;
@@ -281,7 +282,7 @@ ephy_gui_set_default_window_icon (GtkWindow *window)
 		g_warning ("Web browser gnome icon not found");
 	}
 
-	gtk_window_set_icon (GTK_WINDOW (about), icon);
+	gtk_window_set_icon (GTK_WINDOW (window), icon);
 
 	if (icon != NULL)
 	{
