@@ -1,7 +1,7 @@
 /*
  *  Copyright (C) 2001 Matthew Mueller
- *            (C) 2002 Jorn Baayen <jorn@nl.linux.org>
- *	      (C) 2003 Marco Pesenti Gritti <mpeseng@tin.it>
+ *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
+ *  Copyright (C) 2003 Marco Pesenti Gritti <mpeseng@tin.it>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
+ *  $Id: ephy-state.h,v 1.7.16.1 2004/10/08 12:16:39 marco Exp $
  */
 
 #ifndef EPHY_STATE_H
@@ -28,20 +29,26 @@ G_BEGIN_DECLS
 
 typedef enum
 {
+	EPHY_STATE_WINDOW_SAVE_NONE = 0,
 	EPHY_STATE_WINDOW_SAVE_SIZE = 1 << 0,
 	EPHY_STATE_WINDOW_SAVE_POSITION = 1 << 1
 } EphyStateWindowFlags;
-	
+
 void ephy_state_add_window	(GtkWidget *window,
 				 const char *name,
 				 int default_width,
 				 int default_heigth,
+				 gboolean maximize,
 				 EphyStateWindowFlags flags);
 
 void ephy_state_add_paned	(GtkWidget *paned,
 				 const char *name,
 				 int default_width);
 
+void ephy_state_add_expander	(GtkWidget *expander,
+				 const char *name,
+				 gboolean default_state);
+
 void ephy_state_save		(void);
 
 G_END_DECLS
