@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmark-action.c,v 1.46 2004/07/31 11:58:56 chpe Exp $
+ *  $Id: ephy-bookmark-action.c,v 1.47.4.1 2004/09/27 18:11:40 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -433,7 +433,7 @@ remove_from_model (GtkWidget *widget)
 	pos = get_item_position (widget, NULL);
 
 	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
-	model = ephy_bookmarks_get_toolbars_model (bookmarks);
+	model = EGG_TOOLBARS_MODEL (ephy_bookmarks_get_toolbars_model (bookmarks));
 
 	egg_toolbars_model_remove_item (model, 0, pos);
 }
@@ -446,7 +446,7 @@ move_in_model (GtkWidget *widget, int direction)
 	int pos, new_pos;
 
 	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
-	model = ephy_bookmarks_get_toolbars_model (bookmarks);
+	model = EGG_TOOLBARS_MODEL (ephy_bookmarks_get_toolbars_model (bookmarks));
 
 	pos = get_item_position (widget, NULL);
 	new_pos = MAX (0, pos + direction);
@@ -586,14 +586,17 @@ show_context_menu (EphyBookmarkAction *action, GtkWidget *proxy,
 			gtk_get_current_event_time ());
 }
 
-static void
+static gboolean
 popup_menu_cb (GtkWidget *widget, EphyBookmarkAction *action)
 {
 	if (gtk_widget_get_ancestor (widget, EPHY_TYPE_BOOKMARKSBAR))
         {
                 show_context_menu (action, widget,
 				   ephy_gui_menu_position_under_widget);
+		return TRUE;
         }
+
+	return FALSE;
 }
 
 static gboolean
@@ -783,7 +786,7 @@ ephy_bookmark_action_finalize (GObject *object)
 	g_free (eba->priv->location);
 	g_free (eba->priv->icon);
 
-	LOG ("Bookmark action finalized")
+	LOG ("Bookmark action %p finalized", object)
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
