@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-shell.c,v 1.91 2004/08/28 10:22:00 chpe Exp $
+ *  $Id: ephy-shell.c,v 1.91.4.1 2004/09/27 18:11:40 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -678,7 +678,7 @@ ephy_shell_get_toolbars_model (EphyShell *shell, gboolean fullscreen)
 		if (shell->priv->toolbars_model == NULL)
 		{
 			EphyBookmarks *bookmarks;
-			EggToolbarsModel *bookmarksbar_model;
+			GObject *bookmarksbar_model;
 
 			shell->priv->toolbars_model = ephy_toolbars_model_new ();
 
@@ -691,7 +691,7 @@ ephy_shell_get_toolbars_model (EphyShell *shell, gboolean fullscreen)
 
 			/* ok, now we can load the model */
 			ephy_toolbars_model_load
-			(EPHY_TOOLBARS_MODEL (shell->priv->toolbars_model));
+				(EPHY_TOOLBARS_MODEL (shell->priv->toolbars_model));
 		}
 
 		return G_OBJECT (shell->priv->toolbars_model);
