@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: window-commands.c,v 1.131.4.1 2004/09/22 19:15:17 chpe Exp $
+ *  $Id: window-commands.c,v 1.131.4.3 2004/10/08 12:16:39 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -882,6 +882,8 @@ window_cmd_edit_toolbar (GtkAction *action,
         gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
 	gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
 
+	ephy_gui_set_default_window_icon (GTK_WINDOW (dialog));
+
 	editor = egg_toolbar_editor_new (GTK_UI_MANAGER (window->ui_merge), model);
 	egg_toolbar_editor_load_actions (EGG_TOOLBAR_EDITOR (editor),
 					 ephy_file ("epiphany-toolbar.xml"));
@@ -905,9 +907,8 @@ window_cmd_edit_toolbar (GtkAction *action,
 	
 	g_signal_connect (G_OBJECT (dialog), "response",
 			  G_CALLBACK (toolbar_editor_response_cb), model);
-	ephy_state_add_window (dialog,
-			       "toolbar_editor",
-		               500, 330,
+	ephy_state_add_window (dialog, "toolbar_editor",
+		               500, 330, FALSE,
 			       EPHY_STATE_WINDOW_SAVE_SIZE);
 	gtk_widget_show (dialog);
 
